#property indicator_separate_window

#property indicator_minimum -90
#property indicator_maximum 90
#property indicator_level1 45
#property indicator_level2 0
#property indicator_level3 -45

#property indicator_buffers 1
#property indicator_color1 Red

#define SCALING_TO_WINDOW_HIGHT 0
#define SCALING_BY_ATR 1

#define MODE_LRMA 4

int MaxHistoryBarsToCount = 50000;

extern int PeriodsMa = 34;
extern int ShiftMa = 0;
extern string PriceFieldConstants = "Close: 0, Open: 1, High: 2, Low: 3, Median: 4, Typical: 5, Weighted: 6";
extern int PriceFieldMa = 0;
extern string ModeMaConstants = "Simple: 0, Exponential: 1, Smoothed: 2, Linear weighted: 3, Linear regression: 4";
extern int ModeMa = 1;

extern int LookBackBars = 8;
extern int WindowSizeInBars = 80;

extern string ScalingModeConstants = "0: Window hight, 1: ATR(Window bars)";
extern int ScalingMode = 0;
extern double WindowAspectRatio = 1.5;
extern double AtrFactor = 10.0;

double MaAngleBuffer[];

int init() {
   SetIndexStyle(0, DRAW_LINE);
   SetIndexBuffer(0, MaAngleBuffer);
   return(0);
}

int start() {
   int countedBars = IndicatorCounted();
   int emptyLength = MathMin(Bars, MathMax(PeriodsMa + LookBackBars + ShiftMa, WindowSizeInBars) + 1);
   int countFrom = MathMin(Bars - MathMax(countedBars, emptyLength) - 1, MaxHistoryBarsToCount);

   if (countFrom >= 0 && countFrom < Bars) {
      if (countedBars < 1) {
         fillFirstValuesWithEmpty(emptyLength);
      }
      countIndicator(countFrom);
   }
   return(0);
}

int countIndicator(int countFrom) {
   int i;
   double maNow, maLookBack;
   double newLwma, newSma;   
   double winMax, winMin, winHeight;
   double scalingFactorToBars, scalingFactorToPrice;
   double xCoordinateFromScaledBars;
   double yCoordinateFromScaledPrice;
   double countedAngleInDegrees;
   
   for (i = countFrom; i >= 0; i--) {
      maNow = iMA(NULL, 0, PeriodsMa, ShiftMa, ModeMa, PriceFieldMa, i);
      maLookBack = iMA(NULL, 0, PeriodsMa, ShiftMa, ModeMa, PriceFieldMa, i + LookBackBars);
      if (ModeMa == MODE_LRMA) {
         newLwma = iMA(NULL, 0, PeriodsMa, ShiftMa, MODE_LWMA, PriceFieldMa, i);
         newSma = iMA(NULL, 0, PeriodsMa, ShiftMa, MODE_SMA, PriceFieldMa, i);
         maNow = 3.0 * newLwma - 2.0 * newSma;
         newLwma = iMA(NULL, 0, PeriodsMa, ShiftMa, MODE_LWMA, PriceFieldMa, i + LookBackBars);
         newSma = iMA(NULL, 0, PeriodsMa, ShiftMa, MODE_SMA, PriceFieldMa, i + LookBackBars);
         maLookBack = 3.0 * newLwma - 2.0 * newSma;
      }
      winMax = High[ArrayMaximum(High, WindowSizeInBars, i)];
      winMin = Low[ArrayMinimum(Low, WindowSizeInBars, i)];
      
      winHeight = winMax - winMin;
      scalingFactorToBars = 1.0 / WindowSizeInBars;
      scalingFactorToPrice = (1.0 / winHeight) / WindowAspectRatio;
      if (ScalingMode == SCALING_BY_ATR) {
         winHeight = iATR(NULL, 0, WindowSizeInBars, i) * AtrFactor;
         scalingFactorToBars = 1.0 / WindowSizeInBars;
         scalingFactorToPrice = 1.0 / winHeight;
      }
      
      xCoordinateFromScaledBars = LookBackBars * scalingFactorToBars;
      yCoordinateFromScaledPrice = (maNow - maLookBack) * scalingFactorToPrice;
      countedAngleInDegrees = (MathArctan(yCoordinateFromScaledPrice / xCoordinateFromScaledBars) / 3.14159265) * 180.0;
      
      MaAngleBuffer[i] = countedAngleInDegrees;
   }
   return(0);
}

int fillFirstValuesWithEmpty(int emptyLength) {
   int i;
   int fillToIndex = MathMin(Bars - 1 - emptyLength, MaxHistoryBarsToCount - 1 - emptyLength);
   for (i = Bars - 1; i >= fillToIndex; i--) {
      MaAngleBuffer[i] = EMPTY_VALUE;
   }
   return(0);
}

